
#ifndef __ALGORITHMAN__LL1PARSER_TPP
#define __ALGORITHMAN__LL1PARSER_TPP

template<class token> ContextFreeGrammars::LL1Parser<token>::LL1Parser()
{
    this->startsymbol = NULL;
    this->verbose = false;
}

template<class token> ContextFreeGrammars::LL1Parser<token>::~LL1Parser()
{
}


template<class token> void* ContextFreeGrammars::LL1Parser<token>::Parse(ContextFreeGrammars::Tokenizer<token,1>& tokenizer, std::ostream& messages)
{
    Production SProduction;
    SProduction.produced.push_back(this->startsymbol);
    LL1StackFrame* bottomFrame = new LL1StackFrame;
    bottomFrame->production = &SProduction;
    bottomFrame->position = 0;
    parserStack.push(bottomFrame);

    while(!parserStack.empty())
    {
        LL1StackFrame* top = parserStack.top();

        // end of a production reached
        if(top->position >= top->production->produced.size())
        {
            void *productionResult = NULL;
            if(top->production->handler != NULL)
                productionResult = top->production->handler->ProductionApplied(top->subTreeResults);

            parserStack.pop();
            delete top;
            if(parserStack.empty())
                return productionResult;

            top = parserStack.top();
            top->subTreeResults.push_back(productionResult);
            top->position++;
            continue;
        }

        // Terminals
        Terminal<token>* t = dynamic_cast<Terminal<token>*>(top->production->produced[top->position]);
        if(t != NULL)
        {
            if(t->Name != tokenizer.ConsumeToken())
                return NULL; // !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!! FIXME
            
            if(verbose)
                messages << std::string(2*parserStack.size(), ' ') << t->Name << std::endl;

            top->subTreeResults.push_back(NULL);
            top->position++;
            continue;
        }

        // Non-Terminals
        NonTerminal* n = static_cast<NonTerminal*>(top->production->produced[top->position]);
        if(n != NULL) // should always be true
        {
            if(lookaheadTable.find(n) == lookaheadTable.end())
                return NULL; // !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!! FIXME
            

            LL1StackFrame* newFrame = new LL1StackFrame;
            newFrame->position = 0;
            Terminal<token>* peekToken = NULL;

            if(tokenizer.HasToken(0)) // if end of stream: peekToken = NULL causes the production for look-ahead "epsilon" to be selected.
            {
                for(typename std::map<Terminal<token>*, Production*>::iterator tokenFind = lookaheadTable[n].begin(); tokenFind != lookaheadTable[n].end(); tokenFind++)
                    if(tokenFind->first != NULL && tokenFind->first->Token == tokenizer.PeekToken(0))
                        peekToken = tokenFind->first;
                if(peekToken == NULL)
                    return NULL; // !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!! FIXME
            }
            
            newFrame->production = lookaheadTable[n][peekToken];
            
            
            if(verbose)
            {
                messages << std::string(2*parserStack.size(), ' ') << n->Name << " : ";
                for(std::vector<Symbol*>::iterator symb = newFrame->production->produced.begin(); symb != newFrame->production->produced.end(); symb++)
                    messages << (*symb)->Name << " ";
                messages << std::endl;
            }
            
            parserStack.push(newFrame);
            continue;
        }
        
        if(verbose)
            messages << std::string(2*parserStack.size(), ' ') << "ERROR" << std::endl;
        break; // otherwise we'd be stuck in an infinite loop
    }

    return NULL;
}




template<class token> void ContextFreeGrammars::LL1Parser<token>::GenerateCPlusPlusSourceCode(std::ostream& os) const
{
    os << "// Generated by LL1ParserGenerator by Viktor \"AlgorithMan\" Engelmann" << std::endl;
    os << "//                                 http://www.AlgorithMan.de" << std::endl;
    os << "#include<iostream>" << std::endl << std::endl;

    os << "bool verbose = false;" << std::endl << std::endl;

    os << "std::string ConsumeToken(std::string* lookahead, std::istream& is, int depth)" << std::endl;
    os << "{" << std::endl;
    os << "    if(verbose)"<< std::endl;
    os << "        std::cout << std::string(2*depth, ' ') << *lookahead << std::endl;" << std::endl << std::endl;

    os << "    std::string result = *lookahead;" << std::endl;
    os << "    if(is.eof())" << std::endl;
    os << "        *lookahead = \"\";" << std::endl;
    os << "    else" << std::endl;
    os << "    {" << std::endl;
    os << "        is >> *lookahead;" << std::endl;
    os << "        is.peek();" << std::endl;
    os << "        if(is.fail())" << std::endl;
    os << "            *lookahead = \"\";" << std::endl;
    os << "    }" << std::endl << std::endl;

    os << "    return result;" << std::endl;
    os << "}" << std::endl << std::endl;


    // forward declarations
    os << "// forward declarations" << std::endl;
    for(typename std::map<NonTerminal*, std::map<Terminal<token>*,Production*> >::const_iterator n = lookaheadTable.begin(); n != lookaheadTable.end(); n++)
        os << "bool Parse" << n->first->Name << "(std::string* lookahead, std::istream& is, int depth);" << std::endl;
    os << std::endl;


    for(typename std::map<NonTerminal*, std::map<Terminal<token>*,Production*> >::const_iterator l = lookaheadTable.begin(); l != lookaheadTable.end(); l++)
    {
        NonTerminal *n = l->first;
        os << "bool Parse" << n->Name << "(std::string* lookahead, std::istream& is, int depth)" << std::endl << "{" << std::endl;
        os << "    if(verbose)" << std::endl;
        os << "        std::cout << std::string(2*depth, ' ') << \"" << n->Name << ", look-ahead=\\\"\" << *lookahead << \"\\\"\" << std::endl;" << std::endl;

        bool firstproduction = true;
        for(std::vector<Production*>::const_iterator p = n->productions.begin(); p != n->productions.end(); p++)
        {
            os << std::endl << "    // " << n->Name << " :";
            for(std::vector<Symbol*>::const_iterator s = (*p)->produced.begin(); s != (*p)->produced.end(); s++)
                os << " " << (*s)->Name;

            os << std::endl << "    " << (firstproduction ? "" : "else ") << "if(";
            firstproduction = false;


            // reconstruct lookahead set
            std::set<Terminal<token>*> plookahead;
            for(typename std::map<Terminal<token>*,Production*>::const_iterator larev = l->second.begin(); larev != l->second.end(); larev++)
                if(larev->second == *p)
                    plookahead.insert(larev->first);

            bool firstlookahead = true;
            for(typename std::set<Terminal<token>*>::iterator la = plookahead.begin(); la != plookahead.end(); la++)
                if(*la != NULL)
                {
                    os << (firstlookahead ? "" : " || " ) << "*lookahead == \"" << (*la)->Name << "\"";
                    firstlookahead = false;
                }
            if(plookahead.find(NULL) != plookahead.end())
                os << (firstlookahead ? "" : " || " ) << "*lookahead == \"\")" << std::endl;
            else
                os << ")" << std::endl;

            os << "    {" << std::endl;

            for(std::vector<Symbol*>::const_iterator s = (*p)->produced.begin(); s != (*p)->produced.end(); s++)
            {
                Terminal<token>* t = dynamic_cast<Terminal<token>*>(*s);
                if(t != NULL)
                    os << "        if(ConsumeToken(lookahead, is, depth+1) != \"" << t->Name << "\") return false;" << std::endl;
                else
                {
                    NonTerminal* nt = static_cast<NonTerminal*>(*s);
                    os << "        if(!Parse" << nt->Name << "(lookahead, is, depth+1)) return false;" << std::endl;
                }
            }
            os << "        return true;" << std::endl;
            os << "    }" << std::endl;
        }

        os << std::endl << "    return false;" << std::endl;
        os << "}" << std::endl << std::endl;
    }

    os << "int main(int argc, char** argv)" << std::endl;
    os << "{" << std::endl;
    os << "    std::cout << \"LL(1) Parser generated by LL1ParserGenerator by Viktor \\\"AlgorithMan\\\" Engelmann\" << std::endl;" << std::endl;
    os << "    std::cout << \"                                             http://www.AlgorithMan.de\" << std::endl;" << std::endl;
    os << "    std::string lookahead;" << std::endl;
    os << "    std::cin >> lookahead;" << std::endl;
    os << "    if(argc > 1 && std::string(argv[1]) == \"-v\")" << std::endl;
    os << "        verbose = true;" << std::endl;
    os << "    std::cout << \"Parsing stdin...\\n\";" << std::endl;
    if(startsymbol != NULL)
        os << "    bool result = Parse" << startsymbol->Name << "(&lookahead, std::cin, 0);" << std::endl;
    else
        os << "    bool result = false; // a Context Free Grammar without productions produces the empty language => all inputs rejected" << std::endl;
    os << "    std::cout << (result ? \"success\" : \"failed\") << std::endl;" << std::endl;
    os << "    return result ? 0 : 1;" << std::endl;
    os << "}" << std::endl << std::endl;
}



#endif