
#ifndef __ALGORITHMAN__LR0PARSERGENERATOR_TPP
#define __ALGORITHMAN__LR0PARSERGENERATOR_TPP









template<class Token> ContextFreeGrammars::LR0ParserGenerator<Token>::LR0ParserGenerator(const ContextFreeGrammars::ContextFreeGrammar<Token>& cfg)
{
    this->cfg = cfg;
}

template<class Token> ContextFreeGrammars::LR0ParserGenerator<Token>::~LR0ParserGenerator()
{
}

template<class Token> void ContextFreeGrammars::LR0ParserGenerator<Token>::Closure0(std::set<ContextFreeGrammars::LR0Element*>& I)
{
    std::set<LR0Element*> newOnes = I;

    while(newOnes.size() > 0)
    {
        newOnes.clear();

        for(std::set<LR0Element*>::iterator i = I.begin(); i != I.end(); i++)
        {
            LR0Element* e = *i;
            if(e->dotPosition < e->production->produced.size())
            {
                NonTerminal* n = dynamic_cast<NonTerminal*>(e->production->produced[e->dotPosition]);
                if(n != NULL)
                {
                    for(std::vector<Production*>::iterator p = n->productions.begin(); p != n->productions.end(); p++)
                    {
                        bool isNew = true;
                        for(std::set<LR0Element*>::iterator i = I.begin(); isNew && i != I.end(); i++)
                            if((*i)->production == *p && (*i)->dotPosition == 0)
                                isNew = false;
                        for(std::set<LR0Element*>::iterator i = newOnes.begin(); isNew && i != newOnes.end(); i++)
                            if((*i)->production == *p && (*i)->dotPosition == 0)
                                isNew = false;

                        if(isNew)
                            newOnes.insert(new LR0Element(n, *p, 0));
                    }
                }
            }

        }

        I.insert(newOnes.begin(), newOnes.end());
    }
}

template<class Token> std::set<ContextFreeGrammars::LR0Element*> ContextFreeGrammars::LR0ParserGenerator<Token>::Goto0(std::set<ContextFreeGrammars::LR0Element*>& I, ContextFreeGrammars::Symbol *X)
{
    std::set<LR0Element*> result;

    for(std::set<LR0Element*>::iterator i = I.begin(); i != I.end(); i++)
    {
        LR0Element* e = *i;
        if(e->dotPosition < e->production->produced.size())
            if(e->production->produced[e->dotPosition] == X)
                result.insert(new LR0Element(e->nonterminal, e->production, e->dotPosition+1));
    }

    Closure0(result);
    return result;
}



template<class token> int ContextFreeGrammars::LR0ParserGenerator<token>::FindSyntaxAnalysisTableRow(std::set<ContextFreeGrammars::LR0Element*>& state)
{
    int rownumber = 0;
    for(std::vector<std::set<LR0Element*>* >::iterator row = syntaxAnalysisTable.begin(); row != syntaxAnalysisTable.end(); row++, rownumber++)
    {
        if((*row)->size() != state.size()) // speeds it up
            continue;

        bool isEqual = true;

        // test if row <= state
        for(std::set<LR0Element*>::iterator rowit = (*row)->begin(); isEqual && rowit != (*row)->end(); rowit++)
        {
            bool stateContainsRowit = false;
            for(std::set<LR0Element*>::iterator stateit = state.begin(); !stateContainsRowit && stateit != state.end(); stateit++)
                stateContainsRowit = stateContainsRowit || (**rowit == **stateit);
            isEqual = isEqual && stateContainsRowit;
        }

        // test if state <= row
        for(std::set<LR0Element*>::iterator stateit = state.begin(); isEqual && stateit != state.end(); stateit++)
        {
            bool rowContainsStateit = false;
            for(std::set<LR0Element*>::iterator rowit = (*row)->begin(); !rowContainsStateit && rowit != (*row)->end(); rowit++)
                rowContainsStateit = rowContainsStateit || (**rowit == **stateit);
            isEqual = isEqual && rowContainsStateit;
        }

        if(isEqual)
            return rownumber;
    }

    return -1;
}

template<class token> ContextFreeGrammars::LRParser<token>* ContextFreeGrammars::LR0ParserGenerator<token>::Generate()
{
    LRParser<token> *result = new LRParser<token>();

    Production* initialProduction = new Production();
    initialProduction->produced.push_back(cfg.startsymbol);
    LR0Element* initialElement = new LR0Element(cfg.startsymbol, initialProduction, 0);
    std::set<LR0Element*> *I = new std::set<LR0Element*>();
    I->insert(initialElement);
    Closure0(*I);
    syntaxAnalysisTable.push_back(I);

    std::queue<std::set<LR0Element*>*> que;
    que.push(I);
    while(!que.empty())
    {
        std::set<LR0Element*>* front = que.front();
        que.pop();

        result->actionTable.push_back(std::map<token,LRParserAction<token>*>());
        result->jumpTable.push_back(std::map<NonTerminal*,int>());
        result->finalActionTable.push_back(NULL);
        int row = result->actionTable.size() - 1;


        std::cout << "\nState " << row << std::endl;
        for(std::set<LR0Element*>::iterator i = front->begin(); i != front->end(); i++)
        {
            std::cout << (*i)->nonterminal->Name << " -> ";
            for(int j = 0; j < (*i)->production->produced.size(); j++)
            {
                if(j == (*i)->dotPosition)
                    std::cout << ". ";
                std::cout << (*i)->production->produced[j]->Name << " ";
            }
            if((*i)->production->produced.size() == (*i)->dotPosition)
                std::cout << ".";
            std::cout << std::endl;
        }


        // create shift-instructions and jump-targets for reduce-instructions
        for(typename std::map<std::string, Symbol*>::iterator i = cfg.symbols.begin(); i != cfg.symbols.end(); i++)
        {
            Symbol* symb = i->second;
            std::set<LR0Element*>* succ = new std::set<LR0Element*>(Goto0(*front, symb));
            int succSize = succ->size();

            // find row of Goto0(*front, symb) (insert it, if it is not in the table yet)
            int succRow = FindSyntaxAnalysisTableRow(*succ);
            if(succRow < 0 && succSize > 0)
            {
                syntaxAnalysisTable.push_back(succ);
                que.push(succ);
                succRow = syntaxAnalysisTable.size() - 1;
            }
            else
            {
                for(std::set<LR0Element*>::iterator j = succ->begin(); j != succ->end(); j++)
                    delete *j;
                delete succ;
                if(succSize <= 0)
                    continue;
                succ = syntaxAnalysisTable[succRow];
            }


            // create shift-instructions for terminals
            // and reduce-targets for nonterminals
            Terminal<token>* tok = dynamic_cast<Terminal<token>*>(symb);
            if(tok != NULL)
            {
                result->actionTable[row][tok->Token] = new LRParserActionShift<token>(succRow);
            }
            else
            {
                NonTerminal* nterm = static_cast<NonTerminal*>(symb);
                result->jumpTable[row][nterm] = succRow;
            }
        }

        // create reduce-instructions
        for(std::set<LR0Element*>::iterator lr0element = front->begin(); lr0element != front->end(); lr0element++)
        {
            if((*lr0element)->dotPosition < (*lr0element)->production->produced.size()) // incomplete production, cannot reduce this
                continue;

            for(typename std::vector<Terminal<token>*>::iterator t = cfg.terminals.begin(); t != cfg.terminals.end(); t++)
            {
                if(result->actionTable[row].find((*t)->Token) == result->actionTable[row].end())
                {
                    result->actionTable[row][(*t)->Token] = new LRParserActionReduce<token>((*lr0element)->nonterminal, (*lr0element)->production);
                }
                else
                {
                    LRParserAction<token> *prevAction = result->actionTable[row][(*t)->Token];
                    std::cout << (dynamic_cast<LRParserActionShift<token>*>(prevAction) != NULL ? "Shift" : "Reduce")
                              << "/Reduce Conflict in State " << row << " for token " << (*t)->Token << std::endl;
                }
            }

            // create End-Of-Stream action
            if(result->finalActionTable[row] == NULL)
                result->finalActionTable[row] = new LRParserActionReduce<token>((*lr0element)->nonterminal, (*lr0element)->production);
            else
                std::cout << "Reduce/Reduce Conflict in State " << row << " for end-of-stream" << std::endl;
        }
    }



    // Create the accept-instruction
    std::set<LR0Element*>* acceptingState = new std::set<LR0Element*>(Goto0(*I, cfg.startsymbol));
    int acceptingStateRow = FindSyntaxAnalysisTableRow(*acceptingState);
    if(result->finalActionTable[acceptingStateRow] != NULL)
        delete result->finalActionTable[acceptingStateRow];
    result->finalActionTable[acceptingStateRow] = new LRParserAction<token>();



    // free the memory we used
    for(std::vector<std::set<LR0Element*>* >::iterator row = syntaxAnalysisTable.begin(); row != syntaxAnalysisTable.end(); row++)
    {
        for(std::set<LR0Element*>::iterator i = (*row)->begin(); i != (*row)->end(); i++)
            delete *i;
        delete *row;
    }
    syntaxAnalysisTable.clear();
    //delete initialProduction;


    return result;
}

#endif

